# -*- coding: utf-8 -*-

"""
@date: 2020/12/28 上午10:35
@file: key_word.py
@author: zj
@description: 
"""

KEY_OUTPUT = 'output'
KEY_LOSS = 'loss'

KEY_IMGS = 'imgs'
KEY_TARGETS = 'targets'
KEY_CLASSES = 'classes'
